package com.jiuqi.script.rhino;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.script.CompileException;
import com.jiuqi.script.EvalException;
import com.jiuqi.script.IScriptObject;
import com.jiuqi.script.Script;
import com.jiuqi.script.ScriptFactory;
import com.jiuqi.script.ScriptManager;
import com.jiuqi.script.rhino.RhinoScriptFactory;
import com.jiuqi.script.rhino.RhinoScriptObject;
import com.jiuqi.script.rhino.ScriptHelper;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Calendar;
import java.util.Date;
import org.mozilla.javascript.BaseFunction;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.FunctionObject;
import org.mozilla.javascript.IdFunctionObject;
import org.mozilla.javascript.IdScriptableObject;
import org.mozilla.javascript.JavaScriptException;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;

public class RhinoScript implements Script {
  private Context cx;
  
  private Scriptable scope;
  
  static {
    ScriptManager.registerScriptFactory((ScriptFactory)new RhinoScriptFactory());
  }
  
  public RhinoScript() {
    this.cx = Context.enter();
    this.scope = (Scriptable)this.cx.initStandardObjects(null, false);
    regDefaultFunction();
    Context.exit();
  }
  
  public Object evaluate(String script, String sourceName) throws CompileException, EvalException {
    this.cx = Context.enter();
    try {
      Object result = this.cx.evaluateString(this.scope, script, sourceName, 1, null);
      return convertObject(result);
    } catch (JavaScriptException e) {
      throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
    } catch (RhinoException e) {
      throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
    } finally {
      Context.exit();
    } 
  }
  
  public Object call(String name, Object[] args) throws NoSuchMethodException, CompileException, EvalException {
    Object[] param = args;
    if (param == null)
      param = new Object[0]; 
    convertArgument(param);
    this.cx = Context.enter();
    try {
      Function fun = getFunctionFromScope(this.scope, name);
      if (fun != null) {
        Object result = fun.call(this.cx, this.scope, (Scriptable)fun, param);
        return convertObject(result);
      } 
      throw new NoSuchMethodException("没有该函数:" + name);
    } catch (JavaScriptException e) {
      throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
    } catch (RhinoException e) {
      throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
    } finally {
      Context.exit();
    } 
  }
  
  public void compile(String script, String sourceName) throws CompileException {
    this.cx = Context.enter();
    try {
      this.cx.evaluateString(this.scope, script, sourceName, 1, null);
    } catch (RhinoException e) {
      throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
    } finally {
      Context.exit();
    } 
  }
  
  public Object get(String name) {
    Object o = getValueFromScope(this.scope, name);
    if (o instanceof org.mozilla.javascript.UniqueTag || o instanceof org.mozilla.javascript.Undefined)
      return null; 
    return o;
  }
  
  public void put(String name, Object value) {
    putValueToScope(this.scope, name, value);
  }
  
  public Object toJava(Object obj) {
    if (obj == null)
      return null; 
    if (obj instanceof RhinoScriptObject)
      return ((RhinoScriptObject)obj).getScriptObject(); 
    if (!(obj instanceof IdScriptableObject))
      return obj; 
    IdScriptableObject result = (IdScriptableObject)obj;
    if (result.getClassName().equals("Date")) {
      IdFunctionObject v = (IdFunctionObject)ScriptableObject.getProperty((Scriptable)result, "getTime");
      Object m = result.execIdCall(v, Context.getCurrentContext(), 
          this.scope, (Scriptable)result, null);
      long millis = ((Double)m).longValue();
      Calendar c = Calendar.getInstance();
      c.setTimeInMillis(millis);
      return c;
    } 
    return result;
  }
  
  public Object toScript(Object obj) {
    if (obj == null)
      return null; 
    if (obj instanceof IScriptObject)
      return new RhinoScriptObject((IScriptObject)obj); 
    if (obj instanceof IdScriptableObject)
      return obj; 
    if (obj instanceof NativeJavaObject) {
      Object result = ((NativeJavaObject)obj).unwrap();
      if (result instanceof IScriptObject)
        return new RhinoScriptObject((IScriptObject)result); 
      if (result instanceof Calendar) {
        Calendar c = (Calendar)result;
        return Context.getCurrentContext().newObject(this.scope, 
            "Date", new Object[] { new Long(c.getTimeInMillis()) });
      } 
      if (result instanceof Date) {
        Date d = (Date)result;
        return Context.getCurrentContext().newObject(this.scope, 
            "Date", new Object[] { new Long(d.getTime()) });
      } 
    } 
    return obj;
  }
  
  private Function getFunctionFromScope(Scriptable scope, String name) {
    String parentName = name;
    Scriptable parent = scope;
    if (name == null)
      return null; 
    while (name.indexOf(".") != -1) {
      parentName = parentName.substring(0, name.indexOf("."));
      Object object = parent.get(parentName, scope);
      if (object instanceof Scriptable) {
        parent = (Scriptable)object;
      } else {
        return null;
      } 
      name = name.substring(name.indexOf(".") + 1);
    } 
    Object o = parent.get(name, scope);
    if (o instanceof Function)
      return (Function)o; 
    return null;
  }
  
  private Object getValueFromScope(Scriptable scope, String name) {
    String parentName = name;
    Scriptable parent = scope;
    if (name == null)
      return null; 
    while (name.indexOf(".") != -1) {
      parentName = parentName.substring(0, name.indexOf("."));
      Object o = parent.get(parentName, scope);
      if (o instanceof Scriptable) {
        parent = (Scriptable)o;
      } else {
        return null;
      } 
      name = name.substring(name.indexOf(".") + 1);
    } 
    return parent.get(name, scope);
  }
  
  private void putValueToScope(Scriptable scope, String name, Object value) {
    String parentName = name;
    Scriptable parent = scope;
    if (name == null)
      return; 
    while (name.indexOf(".") != -1) {
      parentName = parentName.substring(0, name.indexOf("."));
      Object o = parent.get(parentName, scope);
      if (o instanceof Scriptable) {
        parent = (Scriptable)o;
      } else {
        return;
      } 
      name = name.substring(name.indexOf(".") + 1);
    } 
    if (value instanceof IScriptObject) {
      parent.put(name, scope, new RhinoScriptObject((IScriptObject)value));
    } else {
      parent.put(name, scope, value);
    } 
  }
  
  private void convertArgument(Object[] args) {
    if (args == null)
      return; 
    for (int i = 0; i < args.length; i++) {
      if (args[i] instanceof Calendar) {
        Calendar c = (Calendar)args[i];
        args[i] = this.cx.newObject(this.scope, "Date", 
            new Object[] { new Long(c.getTimeInMillis()) });
      } 
    } 
  }
  
  private Object convertObject(Object result) {
    if (result instanceof ScriptableObject) {
      ScriptableObject o = (ScriptableObject)result;
      if (o.getClassName().equals("CustomScriptObject"))
        return ((RhinoScriptObject)o).getScriptObject(); 
      if (o.getClassName().equals("Date")) {
        BaseFunction v = (BaseFunction)ScriptableObject.getProperty((Scriptable)o, "getTime");
        Object m = v.call(this.cx, this.scope, (Scriptable)o, null);
        long millis = ((Double)m).longValue();
        Calendar c = Calendar.getInstance();
        c.setTimeInMillis(millis);
        return c;
      } 
      return o;
    } 
    if (result instanceof NativeJavaObject) {
      Object o = ((NativeJavaObject)result).unwrap();
      return o;
    } 
    return result;
  }
  
  private void regDefaultFunction() {
    if (this.scope == null)
      return; 
    Class<ScriptHelper> c = ScriptHelper.class;
    ScriptHelper u = new ScriptHelper(this.scope);
    Method[] methods = c.getDeclaredMethods();
    for (int i = 0; i < methods.length; i++) {
      if (!methods[i].getName().equals("getClassName"))
        if (Modifier.isPublic(methods[i].getModifiers()))
          this.scope.put(methods[i].getName(), 
              this.scope, 
              new FunctionObject(methods[i].getName(), 
                methods[i], (Scriptable)u));  
    } 
  }
}
